/* global jest, describe, it, expect*/

'use strict';

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _reactAddons = require('react/addons');

var _reactAddons2 = _interopRequireDefault(_reactAddons);

jest.dontMock('../components/TextInput');
jest.dontMock('../utils/inject-style');

var TestUtils = _reactAddons2['default'].addons.TestUtils;

// Babel would move an import in front of the jest.dontMock. That's why require
// is used instead of import.
var injectStyle = require('../utils/inject-style');
var TextInput = require('../components/TextInput');

describe('TextInput', function () {
  it('should come with default styles', function () {
    var textInput = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(TextInput, null));
    var textareaNode = TestUtils.findRenderedDOMComponentWithTag(textInput, 'textarea');

    expect(textareaNode.props.style).toBeDefined();
    expect(textareaNode.props.style.overflow).toBe('hidden');
  });

  it('should be able to adopt the style of the textInput', function () {
    var textInput = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(TextInput, { style: { color: 'red' } }));
    var textareaNode = TestUtils.findRenderedDOMComponentWithTag(textInput, 'textarea');

    expect(textareaNode.props.style.color).toBe('red');
  });

  it('should calculate its height after initializing', function () {
    var textInput = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(TextInput, null));

    textInput._triggerResize = jest.genMockFunction();
    textInput.componentDidMount();

    expect(textInput._triggerResize.mock.calls.length).toBe(1);
  });

  it('should re-calculate its height after changing (default)', function () {
    var textInput = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(TextInput, null));

    textInput._triggerResize = jest.genMockFunction();

    var textareaNode = TestUtils.findRenderedDOMComponentWithTag(textInput, 'textarea');

    TestUtils.Simulate.change(textareaNode, 'some input text');

    expect(textInput._triggerResize.mock.calls.length).toBe(1);
  });

  it('should re-calculate its height after changing (with value & new lines not allowed)', function () {
    var textInput = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(TextInput, { value: 'some text' }));

    textInput._triggerResize = jest.genMockFunction();

    var textareaNode = TestUtils.findRenderedDOMComponentWithTag(textInput, 'textarea');

    TestUtils.Simulate.change(textareaNode, 'some other text');

    expect(textInput._triggerResize.mock.calls.length).toBe(1);
  });

  it('should re-calculate its height after changing (new lines allowed)', function () {
    var textInput = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(TextInput, { value: 'some text', allowNewLine: true }));

    textInput._triggerResize = jest.genMockFunction();

    var textareaNode = TestUtils.findRenderedDOMComponentWithTag(textInput, 'textarea');

    TestUtils.Simulate.change(textareaNode, 'some other text');

    expect(textInput._triggerResize.mock.calls.length).toBe(1);
  });

  it('should be able to bind onKeyDown', function () {
    var wasPressed = false;

    var textInput = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(TextInput, { onKeyDown: function () {
        return wasPressed = true;
      } }));

    var textareaNode = TestUtils.findRenderedDOMComponentWithTag(textInput, 'textarea');

    TestUtils.Simulate.keyDown(textareaNode, { key: '1' });

    expect(wasPressed).toEqual(true);
  });

  it('should be able to bind onUpdate', function () {
    var wasChanged = false;

    var textInput = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(TextInput, { onUpdate: function () {
        return wasChanged = true;
      } }));

    var textareaNode = TestUtils.findRenderedDOMComponentWithTag(textInput, 'textarea');

    TestUtils.Simulate.change(textareaNode, 'some text');

    expect(wasChanged).toEqual(true);
  });

  it('should be able to provide a valueLink', function () {
    var wasCalled = false;

    var valueLink = {
      requestChange: function requestChange() {
        wasCalled = true;
      },
      value: 'some text'
    };

    var textInput = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(TextInput, { valueLink: valueLink }));

    var textareaNode = TestUtils.findRenderedDOMComponentWithTag(textInput, 'textarea');

    TestUtils.Simulate.change(textareaNode, 'some other text');

    expect(wasCalled).toEqual(true);
  });

  it('should be able to provide a className', function () {
    var textInput = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(TextInput, { className: 'test-me' }));

    var textareaNode = TestUtils.findRenderedDOMComponentWithTag(textInput, 'textarea');
    expect(textareaNode.props.className.indexOf('test-me')).toBeGreaterThan(-1);
  });

  it('should remove the custom styles from the dom when the textInput unmounts', function () {
    injectStyle.removeStyle = jest.genMockFunction();

    var textInput = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(TextInput, null));

    textInput.componentWillUnmount();

    expect(injectStyle.removeStyle.mock.calls.length).toBe(1);
  });
});